/*
  This is globals.h

  Coxeter version 3.0  Copyright (C) 2002 Fokko du Cloux
  See file main.cpp for full copyright notice
*/

#ifndef GLOBALS_H  /* guard against multiple inclusions */
#define GLOBALS_H

// TODO: Check to make sure we need these
#include <iostream>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

namespace coxeter {
  typedef unsigned long Ulong; /* shorthand */
};

#endif
