/*
  This is version.h
  
  Coxeter version 3.0  Copyright (C) 2002 Fokko du Cloux
  Coxeter version 3.1  Copyright (C) 2015 Travis Scrimshaw
  See file main.cpp for full copyright notice
*/

#ifndef VERSION_H  /* guard against multiple inclusions */
#define VERSION_H

namespace version {
  const char* const NAME = "Coxeter";
  const char* const VERSION = "3.1";
};

#endif
